(function(){

/////////////////////////////////////////////////////////////////////////
//                                                                     //
// server/drivers/external-ui-view.js                                  //
//                                                                     //
/////////////////////////////////////////////////////////////////////////
                                                                       //
// Sandstorm - Personal Cloud Sandbox                                  //
// Copyright (c) 2014 Sandstorm Development Group, Inc. and contributors
// All rights reserved.                                                //
//                                                                     //
// Licensed under the Apache License, Version 2.0 (the "License");     //
// you may not use this file except in compliance with the License.    //
// You may obtain a copy of the License at                             //
//                                                                     //
//   http://www.apache.org/licenses/LICENSE-2.0                        //
//                                                                     //
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,   //
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.                                      //
                                                                       //
var Future = Npm.require('fibers/future');                             // 17
var Promise = Npm.require('es6-promise').Promise;                      // 18
var Capnp = Npm.require('capnp');                                      // 19
var Url = Npm.require('url');                                          // 20
var Http = Npm.require('http');                                        // 21
var Https = Npm.require('https');                                      // 22
var ApiSession = Capnp.importSystem('sandstorm/api-session.capnp').ApiSession;
                                                                       //
WrappedUiView = (function () {                                         // 25
  function WrappedUiView(token, proxy) {                               // 26
    babelHelpers.classCallCheck(this, WrappedUiView);                  //
                                                                       //
    // TODO(someday): handle the fact that these proxies will be garbage collected every 2 minutes,
    // even if it's in use.                                            //
    this.token = token;                                                // 29
    this.proxy = proxy;                                                // 30
  }                                                                    //
                                                                       //
  WrappedUiView.prototype.newSession = (function () {                  // 25
    function newSession(userInfo, context, sessionType, sessionParams, retryCount) {
      if (sessionType !== ApiSession.typeId) {                         // 34
        throw new Error('SessionType must be ApiSession.');            // 35
      }                                                                //
                                                                       //
      retryCount = retryCount || 0;                                    // 38
      var _this = this;                                                // 39
                                                                       //
      return this.proxy.keepAlive().then(function () {                 // 41
        // ignore the passed userInfo and instead use the one associated with this
        // token TODO(someday): Merge / intersect the two userInfo objects
        // (especially permissions) rather than only taking the token's user info.
        // This will allow the caller to request only a subset of the permissions
        // granted by the token, which is useful to protect against bugs.
        var session = _this.proxy.uiView.newSession(_this.proxy.userInfo, context, sessionType, sessionParams).session;
        return { session: session };                                   // 49
      })['catch'](function (error) {                                   //
        return _this.proxy.maybeRetryAfterError(error, retryCount).then(function () {
          return _this.newSession(userInfo, context, sessionType, sessionParams, retryCount + 1);
        });                                                            //
      });                                                              //
    }                                                                  //
                                                                       //
    return newSession;                                                 //
  })();                                                                //
                                                                       //
  return WrappedUiView;                                                //
})();                                                                  //
                                                                       //
getWrappedUiViewForToken = function (token) {                          // 58
  var proxyPromise = getProxyForApiToken(token);                       // 59
                                                                       //
  return proxyPromise.then(function (proxy) {                          // 61
    return { view: new WrappedUiView(token, proxy) };                  // 62
  });                                                                  //
};                                                                     //
                                                                       //
ExternalUiView = (function () {                                        // 66
  function ExternalUiView(url, grainId, token) {                       // 67
    babelHelpers.classCallCheck(this, ExternalUiView);                 //
                                                                       //
    this.url = url;                                                    // 68
    this.grainId = grainId;                                            // 69
    this.token = token;                                                // 70
  }                                                                    //
                                                                       //
  ExternalUiView.prototype.newSession = (function () {                 // 66
    function newSession(userInfo, context, sessionType, sessionParams) {
      if (sessionType !== ApiSession.typeId) {                         // 74
        throw new Error('SessionType must be ApiSession.');            // 75
      }                                                                //
                                                                       //
      var options = {};                                                // 78
                                                                       //
      if (this.token) {                                                // 80
        options.headers = {                                            // 81
          authorization: 'Bearer ' + this.token                        // 82
        };                                                             //
      }                                                                //
                                                                       //
      return { session: new Capnp.Capability(new ExternalWebSession(this.url, this.grainId, options), ApiSession) };
    }                                                                  //
                                                                       //
    return newSession;                                                 //
  })();                                                                //
                                                                       //
  return ExternalUiView;                                               //
})();                                                                  //
                                                                       //
var responseCodes = {                                                  // 90
  200: { type: 'content', code: 'ok' },                                // 91
  201: { type: 'content', code: 'created' },                           // 92
  202: { type: 'content', code: 'accepted' },                          // 93
  204: { type: 'noContent', shouldResetForm: false },                  // 94
  205: { type: 'noContent', shouldResetForm: true },                   // 95
                                                                       //
  // Unsupported until something demonstrates need.                    //
  // 206: {type: 'noContent'},                                         //
  // 300: {type: 'redirect'},                                          //
  301: { type: 'redirect', switchToGet: true, isPermanent: true },     // 100
  302: { type: 'redirect', switchToGet: true, isPermanent: false },    // 101
  303: { type: 'redirect', switchToGet: true, isPermanent: false },    // 102
                                                                       //
  // Unsupported until something demonstrates need.                    //
  // 304: {type: 'redirect'},                                          //
  // 305: {type: 'redirect'},                                          //
  307: { type: 'redirect', switchToGet: false, isPermanent: false },   // 107
  308: { type: 'redirect', switchToGet: false, isPermanent: true },    // 108
  400: { type: 'clientError', clientErrorCode: 'badRequest', descriptionHtml: 'Bad Request' },
  403: { type: 'clientError', clientErrorCode: 'forbidden', descriptionHtml: 'Forbidden' },
  404: { type: 'clientError', clientErrorCode: 'notFound', descriptionHtml: 'Not Found' },
  405: { type: 'clientError', clientErrorCode: 'methodNotAllowed', descriptionHtml: 'Method Not Allowed' },
  406: { type: 'clientError', clientErrorCode: 'notAcceptable', descriptionHtml: 'Not Acceptable' },
  409: { type: 'clientError', clientErrorCode: 'conflict', descriptionHtml: 'Conflict' },
  410: { type: 'clientError', clientErrorCode: 'gone', descriptionHtml: 'Gone' },
  413: { type: 'clientError', clientErrorCode: 'requestEntityTooLarge', descriptionHtml: 'Request Entity Too Large' },
  414: { type: 'clientError', clientErrorCode: 'requestUriTooLong', descriptionHtml: 'Request-URI Too Long' },
  415: { type: 'clientError', clientErrorCode: 'unsupportedMediaType', descriptionHtml: 'Unsupported Media Type' },
  418: { type: 'clientError', clientErrorCode: 'imATeapot', descriptionHtml: 'I\'m a teapot' },
  500: { type: 'serverError' },                                        // 120
  501: { type: 'serverError' },                                        // 121
  502: { type: 'serverError' },                                        // 122
  503: { type: 'serverError' },                                        // 123
  504: { type: 'serverError' },                                        // 124
  505: { type: 'serverError' }                                         // 125
};                                                                     //
                                                                       //
ExternalWebSession = (function () {                                    // 128
  function ExternalWebSession(url, grainId, options) {                 // 129
    babelHelpers.classCallCheck(this, ExternalWebSession);             //
                                                                       //
    var parsedUrl = Url.parse(url);                                    // 130
    this.host = parsedUrl.hostname;                                    // 131
    this.port = parsedUrl.port;                                        // 132
    this.protocol = parsedUrl.protocol;                                // 133
    this.grainId = grainId;                                            // 134
    this.options = options || {};                                      // 135
  }                                                                    //
                                                                       //
  ExternalWebSession.prototype.get = (function () {                    // 128
    function get(path, context) {                                      // 138
      return this._requestHelper('GET', path, context);                // 139
    }                                                                  //
                                                                       //
    return get;                                                        //
  })();                                                                //
                                                                       //
  ExternalWebSession.prototype.post = (function () {                   // 128
    function post(path, content, context) {                            // 142
      return this._requestHelper('POST', path, context, content.content, content.mimeType);
    }                                                                  //
                                                                       //
    return post;                                                       //
  })();                                                                //
                                                                       //
  ExternalWebSession.prototype.put = (function () {                    // 128
    function put(path, content, context) {                             // 146
      return this._requestHelper('PUT', path, context, content.content, content.mimeType);
    }                                                                  //
                                                                       //
    return put;                                                        //
  })();                                                                //
                                                                       //
  ExternalWebSession.prototype['delete'] = (function () {              // 128
    function _delete(path, context) {                                  // 150
      return this._requestHelper('DELETE', path, context);             // 151
    }                                                                  //
                                                                       //
    return _delete;                                                    //
  })();                                                                //
                                                                       //
  // TODO(someday): implement streaming and websockets for ExternalWebSession
  // postStreaming(path, mimeType, context) {                          //
  // }                                                                 //
                                                                       //
  // putStreaming(path, mimeType, context) {                           //
  // }                                                                 //
                                                                       //
  // openWebSocket(path, context, protocol, clientStream) {            //
  // }                                                                 //
                                                                       //
  ExternalWebSession.prototype._requestHelper = (function () {         // 128
    function _requestHelper(method, path, context, content, contentType) {
      var _this = this;                                                // 165
      var session = _this;                                             // 166
      return new Promise(function (resolve, reject) {                  // 167
        var options = _.clone(session.options);                        // 168
        options.headers = options.headers || {};                       // 169
        options.path = path;                                           // 170
        options.method = method;                                       // 171
        if (contentType) {                                             // 172
          options.headers['content-type'] = contentType;               // 173
        }                                                              //
                                                                       //
        // set accept header                                           //
        if ('accept' in context) {                                     // 177
          options.headers.accept = context.accept.map(function (acceptedType) {
            return acceptedType.mimeType + '; ' + acceptedType.qValue;
          }).join(', ');                                               //
        } else if (!('accept' in options.headers)) {                   //
          options.headers.accept = '*/*';                              // 182
        }                                                              //
                                                                       //
        // set cookies                                                 //
        if (context.cookies && context.cookies.length > 0) {           // 186
          options.headers.cookies = options.headers.cookies || '';     // 187
          context.cookies.forEach(function (keyVal) {                  // 188
            options.headers.cookies += keyVal.key + '=' + keyVal.val + ',';
          });                                                          //
          options.headers.cookies = options.headers.cookies.slice(0, -1);
        }                                                              //
                                                                       //
        options.host = session.host;                                   // 194
        options.port = session.port;                                   // 195
                                                                       //
        var requestMethod = Http.request;                              // 197
        if (session.protocol === 'https:') {                           // 198
          requestMethod = Https.request;                               // 199
        }                                                              //
                                                                       //
        req = requestMethod(options, function (resp) {                 // 202
          var buffers = [];                                            // 203
          var statusInfo = responseCodes[resp.statusCode];             // 204
                                                                       //
          var rpcResponse = {};                                        // 206
                                                                       //
          switch (statusInfo.type) {                                   // 208
            case 'content':                                            // 209
              resp.on('data', function (buf) {                         // 210
                buffers.push(buf);                                     // 211
              });                                                      //
                                                                       //
              resp.on('end', function () {                             // 214
                var content = {};                                      // 215
                rpcResponse.content = content;                         // 216
                                                                       //
                content.statusCode = statusInfo.code;                  // 218
                if ('content-encoding' in resp.headers) content.encoding = resp.headers['content-encoding'];
                if ('content-language' in resp.headers) content.language = resp.headers['content-language'];
                if ('content-type' in resp.headers) content.language = resp.headers['content-type'];
                if ('content-disposition' in resp.headers) {           // 222
                  var disposition = resp.headers['content-disposition'];
                  var parts = disposition.split(';');                  // 224
                  if (parts[0].toLowerCase().trim() === 'attachment') {
                    parts.forEach(function (part) {                    // 226
                      var splitPart = part.split('=');                 // 227
                      if (splitPart[0].toLowerCase().trim() === 'filename') {
                        content.disposition = { download: splitPart[1].trim() };
                      }                                                //
                    });                                                //
                  }                                                    //
                }                                                      //
                                                                       //
                content.body = {};                                     // 235
                content.body.bytes = Buffer.concat(buffers);           // 236
                                                                       //
                resolve(rpcResponse);                                  // 238
              });                                                      //
              break;                                                   // 240
            case 'noContent':                                          // 240
              var noContent = {};                                      // 242
              rpcResponse.noContent = noContent;                       // 243
              noContent.setShouldResetForm = statusInfo.shouldResetForm;
              resolve(rpcResponse);                                    // 245
              break;                                                   // 246
            case 'redirect':                                           // 246
              var redirect = {};                                       // 248
              rpcResponse.redirect = redirect;                         // 249
              redirect.isPermanent = statusInfo.isPermanent;           // 250
              redirect.switchToGet = statusInfo.switchToGet;           // 251
              if ('location' in resp.headers) redirect.location = resp.headers.location;
              resolve(rpcResponse);                                    // 253
              break;                                                   // 254
            case 'clientError':                                        // 254
              var clientError = {};                                    // 256
              rpcResponse.clientError = clientError;                   // 257
              clientError.statusCode = statusInfo.clientErrorCode;     // 258
              clientError.descriptionHtml = statusInfo.descriptionHtml;
              resolve(rpcResponse);                                    // 260
              break;                                                   // 261
            case 'serverError':                                        // 261
              var serverError = {};                                    // 263
              rpcResponse.serverError = serverError;                   // 264
              clientError.descriptionHtml = statusInfo.descriptionHtml;
              resolve(rpcResponse);                                    // 266
              break;                                                   // 267
            default:                                                   // 267
              // ???                                                   //
              err = new Error('Invalid status code ' + resp.statusCode + ' received in response.');
              reject(err);                                             // 270
              break;                                                   // 271
          }                                                            // 271
        });                                                            //
                                                                       //
        req.on('error', function (e) {                                 // 275
          reject(e);                                                   // 276
        });                                                            //
                                                                       //
        req.setTimeout(15000, function () {                            // 279
          req.abort();                                                 // 280
          err = new Error('Request timed out.');                       // 281
          err.kjType = 'overloaded';                                   // 282
          reject(err);                                                 // 283
        });                                                            //
                                                                       //
        if (content) {                                                 // 286
          req.end(content);                                            // 287
        } else {                                                       //
          req.end();                                                   // 289
        }                                                              //
      });                                                              //
    }                                                                  //
                                                                       //
    return _requestHelper;                                             //
  })();                                                                //
                                                                       //
  return ExternalWebSession;                                           //
})();                                                                  //
/////////////////////////////////////////////////////////////////////////

}).call(this);

//# sourceMappingURL=external-ui-view.js.map
