(function(){

/////////////////////////////////////////////////////////////////////////
//                                                                     //
// shared/shell.js                                                     //
//                                                                     //
/////////////////////////////////////////////////////////////////////////
                                                                       //
// Sandstorm - Personal Cloud Sandbox                                  //
// Copyright (c) 2014 Sandstorm Development Group, Inc. and contributors
// All rights reserved.                                                //
//                                                                     //
// Licensed under the Apache License, Version 2.0 (the "License");     //
// you may not use this file except in compliance with the License.    //
// You may obtain a copy of the License at                             //
//                                                                     //
//   http://www.apache.org/licenses/LICENSE-2.0                        //
//                                                                     //
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,   //
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.                                      //
                                                                       //
// This file implements the common shell components such as the top bar.
// It also covers the root page.                                       //
                                                                       //
var getNamesFromIdentityIds = function (identityIds) {                 // 20
  check(identityIds, [String]);                                        // 21
  if (identityIds.length === 0) {                                      // 22
    return [];                                                         // 23
  }                                                                    //
                                                                       //
  var identities = Meteor.users.find({ _id: { $in: identityIds } });   // 26
  return identities.map(function (identity) {                          // 28
    return { name: identity.profile.name };                            // 29
  });                                                                  //
};                                                                     //
                                                                       //
browseHome = function () {                                             // 32
  Router.go("root");                                                   // 33
};                                                                     //
                                                                       //
if (Meteor.isClient) {                                                 // 36
  getOrigin = function () {                                            // 37
    return document.location.protocol + "//" + document.location.host;
  };                                                                   //
                                                                       //
  // Subscribe to basic grain information first and foremost, since    //
  // without it we might e.g. redirect to the wrong place on login.    //
  globalSubs = [Meteor.subscribe("grainsMenu"), Meteor.subscribe("userPackages"), Meteor.subscribe("devPackages"), Meteor.subscribe("credentials"), Meteor.subscribe("accountIdentities")];
                                                                       //
  Tracker.autorun(function () {                                        // 51
    var me = Meteor.user();                                            // 52
    if (me) {                                                          // 53
      if (me.profile) {                                                // 54
        Meteor.subscribe("identityProfile", me._id);                   // 55
      }                                                                //
      if (me.loginIdentities) {                                        // 57
        me.loginIdentities.forEach(function (identity) {               // 58
          Meteor.subscribe("identityProfile", identity.id);            // 59
        });                                                            //
      }                                                                //
      if (me.nonloginIdentities) {                                     // 62
        me.nonloginIdentities.forEach(function (identity) {            // 63
          Meteor.subscribe("identityProfile", identity.id);            // 64
        });                                                            //
      }                                                                //
    }                                                                  //
  });                                                                  //
}                                                                      //
                                                                       //
if (Meteor.isServer) {                                                 // 71
  // Set up browser policy.                                            //
  //                                                                   //
  // Note that by default (when the browser-policy package is added), Content-Security-Policy will
  // already be set to same-origin-only for everything except XHR and WebSocket. Eval is disabled,
  // but inline script are enabled.                                    //
  //                                                                   //
  // TODO(security): Consider disallowing inline scripts. Currently this forces Meteor to do an
  //   extra round trip on startup to fetch server settings. That seems like something that could
  //   be fixed in Meteor (e.g. embed the settings as JSON text rather than script). Startup time
  //   is incredibly important, and Meteor's templating system (which we use to render all our
  //   HTML) already does a good job of protecting us from script injection, so right now I think
  //   we should favor avoiding the round trip.                        //
  //                                                                   //
  // TODO(someday): Detect when an app tries to navigate off-site using CSP's violation reporting
  //   feature. Ask the user whether they want to open the link in a new tab. This is an annoying
  //   prompt, but if we just open the tab directly then apps can trivially leak by opening tabs
  //   with encoded URLs to an evil server. Although, this attack would be very detectable to the
  //   user, so maybe it's not a big deal...                           //
                                                                       //
  BrowserPolicy.framing.disallow(); // Disallow framing of the UI.     // 91
  BrowserPolicy.content.allowFrameOrigin(getWildcardOrigin());         // 92
                                                                       //
  // Allow anything to be loaded from the static asset host.           //
  var Url = Npm.require("url");                                        // 95
  var staticAssetHost = Url.parse(process.env.ROOT_URL).protocol + "//" + globalDb.makeWildcardHost("static");
  BrowserPolicy.content.allowImageOrigin(staticAssetHost);             // 98
  BrowserPolicy.content.allowScriptOrigin(staticAssetHost);            // 99
  BrowserPolicy.content.allowFontOrigin(staticAssetHost);              // 100
  BrowserPolicy.content.allowConnectOrigin(staticAssetHost);           // 101
                                                                       //
  Meteor.publish("grainsMenu", function () {                           // 103
    if (this.userId) {                                                 // 104
      if (Meteor.settings["public"].quotaEnabled) {                    // 105
        // Hack: Fire off an asynchronous update to the user's storage usage whenever they open the
        //   front page.                                               //
        // TODO(someday): Implement the ability to reactively subscribe to storage usage from the
        //   back-end?                                                 //
        var userId = this.userId;                                      // 110
        globalBackend.cap().getUserStorageUsage(userId).then(function (results) {
          inMeteor(function () {                                       // 112
            Meteor.users.update(userId, { $set: { storageUsage: parseInt(results.size) } });
          });                                                          //
        })["catch"](function (err) {                                   //
          if (err.kjType !== "unimplemented") {                        // 116
            console.error(err.stack);                                  // 117
          }                                                            //
        });                                                            //
      }                                                                //
      var identityIds = SandstormDb.getUserIdentityIds(globalDb.getUser(this.userId));
      return [UserActions.find({ userId: this.userId }), Grains.find({ userId: this.userId }), ApiTokens.find({ 'owner.user.identityId': { $in: identityIds } })];
    } else {                                                           //
      return [];                                                       // 128
    }                                                                  //
  });                                                                  //
                                                                       //
  Meteor.publish("sessions", function (sessionId) {                    // 132
    // sessionId itself should be secret enough, but they are also not meant to be shared, so as
    // a backup we only publish the session to its owner. Note that `userId` can be null if the
    // user is not logged in or is using incognito mode.               //
    check(sessionId, String);                                          // 136
    return Sessions.find({ _id: sessionId, $or: [{ userId: this.userId }, { userId: null }] });
  });                                                                  //
                                                                       //
  Meteor.publish("devPackages", function () {                          // 140
    return DevPackages.find();                                         // 141
  });                                                                  //
                                                                       //
  Meteor.publish("notifications", function () {                        // 144
    return Notifications.find({ userId: this.userId }, { fields: { timestamp: 1, text: 1, grainId: 1, userId: 1, isUnread: 1, appUpdates: 1,
        admin: 1, referral: 1 } });                                    // 147
  });                                                                  //
                                                                       //
  Meteor.publish("notificationGrains", function (notificationIds) {    // 151
    // Since publishes can't be reactive, we leave it to the client to subscribe to both
    // "notifications" and "notificationGrains" reactively.            //
    check(notificationIds, [String]);                                  // 154
    var notifications = Notifications.find({ _id: { $in: notificationIds }, userId: this.userId }, { fields: { grainId: 1 } });
                                                                       //
    var grainIds = notifications.map(function (row) {                  // 158
      return row.grainId;                                              // 159
    });                                                                //
    return Grains.find({ _id: { $in: grainIds } }, { fields: { title: 1 } });
  });                                                                  //
                                                                       //
  Meteor.publish("hasUsers", function () {                             // 164
    // Publish pseudo-collection which tells the client if there are any users at all.
    //                                                                 //
    // TODO(cleanup):  This seems overcomplicated.  Does Meteor have a better way?
    var cursor = Meteor.users.find();                                  // 168
    var self = this;                                                   // 169
    if (cursor.count() > 0) {                                          // 170
      self.added("hasUsers", "hasUsers", { hasUsers: true });          // 171
    } else {                                                           //
      var handle = cursor.observeChanges({                             // 173
        added: function (id) {                                         // 174
          self.added("hasUsers", "hasUsers", { hasUsers: true });      // 175
          handle.stop();                                               // 176
          handle = null;                                               // 177
        }                                                              //
      });                                                              //
      self.onStop(function () {                                        // 180
        if (handle) handle.stop();                                     // 181
      });                                                              //
    }                                                                  //
    self.ready();                                                      // 184
  });                                                                  //
                                                                       //
  Meteor.publish("referralInfoPseudo", function () {                   // 187
    // This publishes a pseudo-collection called referralInfo whose documents have the following
    // form:                                                           //
    //                                                                 //
    // - id: (String) same as the User._id of an identity this user has referred
    // - name: (String) the profile.name from that identity            //
    // - completed: (Boolean) if this referral is complete             //
                                                                       //
    //  If the user is not logged in, then we have no referralInfo.    //
    if (!this.userId) {                                                // 196
      return [];                                                       // 197
    }                                                                  //
                                                                       //
    // Implementation note:                                            //
    //                                                                 //
    // This pseudo-collection is populated very differently for (1) the completed: false case versus
    // the (2) completed: true case.                                   //
                                                                       //
    // Case 1. Publish information about not-yet-complete referrals.   //
    var self = this;                                                   // 206
    var notCompletedReferralIdentitiesCursor = Meteor.users.find({ referredBy: this.userId,
      "profile.name": { $exists: true } }, { fields: {                 // 209
        "_id": 1,                                                      // 211
        "referredBy": 1,                                               // 212
        "profile.name": 1 } });                                        // 213
    var notCompletedReferralIdentitiesHandle = notCompletedReferralIdentitiesCursor.observeChanges({
      // The added function gets called with the id of Bob when Alice refers Bob.
      added: function (id, fields) {                                   // 216
        self.added("referralInfo", id, { name: fields.profile.name, completed: false });
      },                                                               //
      // The removed function gets called when Bob is no longer an uncompleted referral.  Note that
      // this will get more complicated once we support sending completed referrals to the client.
      removed: function (id) {                                         // 221
        self.removed("referralInfo", id);                              // 222
      },                                                               //
      // The modified function gets called when Bob's profile.name changed.
      modified: function (id, fields) {                                // 225
        self.modified("referralInfo", id, { name: fields.profile.name, completed: false });
      } });                                                            //
                                                                       //
    // Case 2. Handle completed referrals.                             //
    //                                                                 //
    // - Do a query for the current list of completed identities.      //
    //                                                                 //
    // - Every time we see a new such identity, we create a query that watches that one identity in
    //   case its profile.name changes.                                //
    //                                                                 //
    // - Also watch the first query, since the list of completed identities might change.
    var handleForProfileNameByIdentityId = {};                         // 237
    var stopWatchingAllIdentities = function () {                      // 238
      Object.keys(handleForProfileNameByIdentityId).forEach(function (identityId) {
        stopWatchingIdentity(identityId);                              // 240
      });                                                              //
    };                                                                 //
    var stopWatchingIdentity = function (identityId) {                 // 243
      var handleForProfileName = handleForProfileNameByIdentityId[identityId];
      if (handleForProfileName) {                                      // 245
        self.removed("referralInfo", identityId);                      // 246
        handleForProfileName.stop();                                   // 247
        // delete is safe because we iterate across `Object.keys()` which returns a copy.
        delete handleForProfileNameByIdentityId[identityId];           // 249
      }                                                                //
    };                                                                 //
    var watchIdentityAndPublishReferralSuccess = function (identityId) {
      var handleForProfileName = handleForProfileNameByIdentityId[identityId];
      if (handleForProfileName) {                                      // 254
        return;                                                        // 255
      }                                                                //
                                                                       //
      handleForProfileName = Meteor.users.find({ _id: identityId }, { fields: {
          "profile.name": 1 } }).observeChanges({                      // 261
        added: function (id, fields) {                                 // 262
          self.added("referralInfo", id, { name: fields.profile.name, completed: true });
        },                                                             //
        changed: function (id, fields) {                               // 265
          self.changed("referralInfo", id, { name: fields.profile.name, completed: true });
        },                                                             //
        removed: function (id) {                                       // 268
          stopWatchingIdentity(id);                                    // 269
        } });                                                          //
      handleForProfileNameByIdentityId[identityId] = handleForProfileName;
    };                                                                 //
                                                                       //
    var completedIdentityIdsHandle = Meteor.users.find({ _id: this.userId,
      referredIdentityIds: { $exists: true } }, { fields: {            // 276
        referredIdentityIds: true } }).observeChanges({                // 278
      // `added` gets called when a user gets their first completed referral.
      added: function (id, fields) {                                   // 280
        for (var i = 0; i < fields.referredIdentityIds.length; i++) {  // 281
          // Unconditionally mark these as successful referrals and start watching.
          watchIdentityAndPublishReferralSuccess(fields.referredIdentityIds[i]);
        }                                                              //
      },                                                               //
      // `changed` gets called when a user adds/removes referredIdentityIds, usually when a
      // referral becomes complete.                                    //
      changed: function (id, fields) {                                 // 289
        // Two major tasks.                                            //
        //                                                             //
        // 1. Look for identityIds to unsubscribe from & send removed notices to the client.
        //                                                             //
        // 2. Look for identityIds to subscribe to.                    //
                                                                       //
        // Task 1. Unsubscribe where needed.                           //
        var referredIdentityIdsAsObject = {};                          // 297
        fields.referredIdentityIds.forEach(function (i) {              // 298
          referredIdentityIdsAsObject[i] = true;                       // 298
        });                                                            //
        Object.keys(handleForProfileNameByIdentityId).forEach(function (identityId) {
          // If the handle doesn't show up in the new list of referredIdentityIds, then remove
          // info from the client & stop it on the server & make it null.
          var handleForProfileName = handleForProfileNameByIdentityId[identityId];
          if (referredIdentityIdsAsObject.hasOwnProperty(identityId)) {
            stopWatchingIdentity(identityId);                          // 304
          }                                                            //
        });                                                            //
                                                                       //
        // Task 2. Subscribe where needed.                             //
        for (var i = 0; i < fields.referredIdentityIds.length; i++) {  // 309
          // The watch... function will avoid double-creating subscriptions, so this is safe.
          watchIdentityAndPublishReferralSuccess(fields.referredIdentityIds[i]);
        }                                                              //
      },                                                               //
      // `removed` gets called when a User suddenly has no referredIdentityIds.
      removed: function () {                                           // 315
        // Remove all data from client; stop all handles.              //
        stopWatchingAllIdentities();                                   // 317
      } });                                                            //
                                                                       //
    // With cases 1 and 2 handled, register a cleanup function, then declare victory.
    self.onStop(function () {                                          // 321
      stopWatchingAllIdentities();                                     // 322
      notCompletedReferralIdentitiesHandle.stop();                     // 323
      completedIdentityIdsHandle.stop();                               // 324
    });                                                                //
                                                                       //
    self.ready();                                                      // 327
  });                                                                  //
                                                                       //
  Meteor.publish("backers", function () {                              // 331
    var backers = Assets.getText("backers.txt");                       // 332
    var self = this;                                                   // 333
    var anonCount = 0;                                                 // 334
    var counter = 0;                                                   // 335
                                                                       //
    backers.split("\n").forEach(function (name) {                      // 337
      name = name.trim();                                              // 338
      if (name === "") {                                               // 339
        ++anonCount;                                                   // 340
      } else {                                                         //
        self.added("backers", counter++, { name: name });              // 342
      }                                                                //
    });                                                                //
                                                                       //
    // Text file ends in \n but that shouldn't count.                  //
    --anonCount;                                                       // 347
                                                                       //
    self.added("backers", "anonymous", { count: anonCount - 1 });      // 349
                                                                       //
    self.ready();                                                      // 351
  });                                                                  //
}                                                                      //
                                                                       //
if (Meteor.isClient) {                                                 // 355
  HasUsers = new Mongo.Collection("hasUsers"); // dummy collection defined above
  Backers = new Mongo.Collection("backers"); // pseudo-collection defined above
  ReferralInfo = new Meteor.Collection("referralInfo"); // pseudo-collection
                                                                       //
  if (Meteor.settings["public"].quotaEnabled) {                        // 360
    window.testDisableQuotaClientSide = function () {                  // 361
      Meteor.settings["public"].quotaEnabled = false;                  // 362
    };                                                                 //
  }                                                                    //
                                                                       //
  Router.onRun(function () {                                           // 366
    // Close menus and popups any time we navigate.                    //
    globalTopbar.reset();                                              // 368
    this.next();                                                       // 369
  });                                                                  //
                                                                       //
  Template.layout.events({                                             // 372
    "click a": function (event) {                                      // 373
      // Close menus if a navigation link is clicked. Usually the Router.onRun(), above, will also
      // execute, but it will not in the case where the link points to the current page, yet we'd
      // really still like for the menus to close in such cases.       //
      if (!event.isDefaultPrevented()) {                               // 377
        globalTopbar.reset();                                          // 378
      }                                                                //
    }                                                                  //
  });                                                                  //
                                                                       //
  Template.about.helpers({                                             // 383
    setDocumentTitle: function () {                                    // 384
      document.title = "About · Sandstorm";                            // 385
    }                                                                  //
  });                                                                  //
                                                                       //
  Template.referrals.helpers({                                         // 389
    setDocumentTitle: function () {                                    // 390
      document.title = "Referral Program · Sandstorm";                 // 391
    }                                                                  //
  });                                                                  //
                                                                       //
  Template.body.onRendered(function () {                               // 396
    // If we're on iOS, set a class name on <body> so we can use CSS styles to work around mobile
    // Safari's ridiculous iframe rendering behavior.                  //
    //                                                                 //
    // Note that this can't be done as a template helper because the <body> tag cannot have
    // attributes determined by helpers. This appears to be a Meteor bug related to the fact that
    // the <body> tag is not inside a <template>, but rather is itself equivalent to a <template>.
    if (/iPad|iPhone|iPod/.test(navigator.userAgent)) {                // 403
      document.body.className = "ios";                                 // 404
    }                                                                  //
  });                                                                  //
                                                                       //
  var MONTHS = ["Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"];
                                                                       //
  var formatInCountdown = function (template, countdownDatetime) {     // 410
    var days = 0;                                                      // 411
    var hours = 0;                                                     // 412
    var minutes = 0;                                                   // 413
    var seconds = 0;                                                   // 414
    var diff = countdownDatetime.getTime() - Date.now();               // 415
                                                                       //
    var units = {                                                      // 417
      day: 86400000,                                                   // 418
      hour: 3600000,                                                   // 419
      minute: 60000,                                                   // 420
      second: 1000                                                     // 421
    };                                                                 //
                                                                       //
    for (var unit in babelHelpers.sanitizeForInObject(units)) {        // 424
      // If it's more than one full unit away, then we'll print in terms of this unit. This does
      // mean that we write e.g. "1 minute" for the whole range between 2 minutes and 1 minute, but
      // whatever, this is typical of these sorts of displays.         //
      if (diff >= units[unit]) {                                       // 428
        var count = Math.floor(diff / units[unit]);                    // 429
        // Update next 1ms after the point where `count` would change.
        setTopBarTimeout(template, diff - count * units[unit] + 1);    // 431
        return {                                                       // 432
          text: "in " + count + " " + unit + (count > 1 ? "s" : ""),   // 433
          className: "countdown-" + unit                               // 434
        };                                                             //
      }                                                                //
    }                                                                  //
                                                                       //
    // We're within a second of the countdown, or past it.             //
    if (diff < -3600000) {                                             // 440
      // Notification appears stale.                                   //
      return null;                                                     // 442
    } else {                                                           //
      setTopBarTimeout(template, diff + 3600001);                      // 444
      return { text: "any moment", className: "countdown-now" };       // 445
    }                                                                  //
  };                                                                   //
                                                                       //
  var formatAccountExpires = function () {                             // 449
    var expires = Meteor.user().expires;                               // 450
    return expires && expires.toLocaleTimeString() || null;            // 451
  };                                                                   //
                                                                       //
  var formatAccountExpiresIn = function (template, currentDatetime) {  // 454
    // TODO(someday): formatInCountdown will set the interval to match account expiration time, and
    // completely overwrite the previous interval for $IN_COUNTDOWN    //
    var user = Meteor.user() || {};                                    // 457
    var expires = user.expires || null;                                // 458
    if (!expires) {                                                    // 459
      return null;                                                     // 460
    } else {                                                           //
      return formatInCountdown(template, expires, currentDatetime);    // 462
    }                                                                  //
  };                                                                   //
                                                                       //
  Template.layout.onCreated(function () {                              // 466
    this.timer = new Tracker.Dependency();                             // 467
    var resizeTracker = this.resizeTracker = new Tracker.Dependency();
    this.resizeFunc = function () {                                    // 469
      resizeTracker.changed();                                         // 470
    };                                                                 //
    window.addEventListener("resize", this.resizeFunc, false);         // 472
  });                                                                  //
                                                                       //
  var setTopBarTimeout = function (template, delay) {                  // 475
    Meteor.clearTimeout(template.timeout);                             // 476
    template.timeout = Meteor.setTimeout(function () {                 // 477
      template.timer.changed();                                        // 478
    }, delay);                                                         //
                                                                       //
    // Make sure we re-run when the timeout triggers.                  //
    template.timer.depend();                                           // 482
  };                                                                   //
                                                                       //
  Template.layout.onDestroyed(function () {                            // 485
    Meteor.clearTimeout(this.timeout);                                 // 486
    window.removeEventListener("resize", this.resizeFunc, false);      // 487
  });                                                                  //
                                                                       //
  Template.referrals.helpers({                                         // 490
    isPaid: function () {                                              // 491
      return Meteor.user() && Meteor.user().plan && Meteor.user().plan !== "free";
    },                                                                 //
    notYetCompleteReferralNames: function () {                         // 494
      return ReferralInfo.find({ completed: false });                  // 495
    },                                                                 //
    completeReferralNames: function () {                               // 497
      return ReferralInfo.find({ completed: true });                   // 498
    }                                                                  //
  });                                                                  //
                                                                       //
  var determineAppName = function (grainId) {                          // 502
    // Returns:                                                        //
    //                                                                 //
    // - The current app title, if we can determine it, or             //
    //                                                                 //
    // - The empty string "", if we can't determine the current app title.
    var params = "";                                                   // 508
                                                                       //
    // Try our hardest to find the package's name, falling back on the default if needed.
    if (grainId) {                                                     // 511
      var grain = Grains.findOne({ _id: grainId });                    // 512
      if (grain && grain.packageId) {                                  // 513
        var thisPackage = Packages.findOne({ _id: grain.packageId });  // 514
        if (thisPackage) {                                             // 515
          params = SandstormDb.appNameFromPackage(thisPackage);        // 516
        }                                                              //
      }                                                                //
    }                                                                  //
                                                                       //
    return params;                                                     // 521
  };                                                                   //
                                                                       //
  var billingPromptState = new ReactiveVar(null);                      // 524
                                                                       //
  var showBillingPrompt = function (reason, next) {                    // 526
    billingPromptState.set({                                           // 527
      reason: reason,                                                  // 528
      db: globalDb,                                                    // 529
      topbar: globalTopbar,                                            // 530
      accountsUi: globalAccountsUi,                                    // 531
      onComplete: function () {                                        // 532
        billingPromptState.set(null);                                  // 533
        if (next) next();                                              // 534
      }                                                                //
    });                                                                //
  };                                                                   //
                                                                       //
  var ifQuotaAvailable = function (next) {                             // 539
    var reason = isUserOverQuota(Meteor.user());                       // 540
    if (reason) {                                                      // 541
      if (window.BlackrockPayments) {                                  // 542
        showBillingPrompt(reason, function () {                        // 543
          // If the user successfully raised their quota, continue the operation.
          if (!isUserOverQuota(Meteor.user())) {                       // 545
            next();                                                    // 546
          }                                                            //
        });                                                            //
      } else {                                                         //
        alert("You are out of storage space. Please delete some things and try again.");
      }                                                                //
    } else {                                                           //
      next();                                                          // 553
    }                                                                  //
  };                                                                   //
                                                                       //
  ifPlanAllowsCustomApps = function (next) {                           // 557
    if (globalDb.isDemoUser() || globalDb.isUninvitedFreeUser()) {     // 558
      if (window.BlackrockPayments) {                                  // 559
        showBillingPrompt("customApp", function () {                   // 560
          // If the user successfully chose a plan, continue the operation.
          if (!globalDb.isDemoUser() && !globalDb.isUninvitedFreeUser()) {
            next();                                                    // 563
          }                                                            //
        });                                                            //
      } else {                                                         //
        alert("Sorry, demo users cannot upload custom apps.");         // 567
      }                                                                //
    } else {                                                           //
      next();                                                          // 570
    }                                                                  //
  };                                                                   //
                                                                       //
  globalQuotaEnforcer = {                                              // 574
    ifQuotaAvailable: ifQuotaAvailable,                                // 575
    ifPlanAllowsCustomApps: ifPlanAllowsCustomApps                     // 576
  };                                                                   //
                                                                       //
  var isDemoExpired = function () {                                    // 579
    var user = Meteor.user();                                          // 580
    if (!user) return false;                                           // 581
    var expires = user.expires;                                        // 582
    if (!expires) return false;                                        // 583
    expires = expires.getTime() - Date.now();                          // 584
    if (expires <= 0) return true;                                     // 585
    var comp = Tracker.currentComputation;                             // 586
    if (expires && comp) {                                             // 587
      Meteor.setTimeout(comp.invalidate.bind(comp), expires);          // 588
    }                                                                  //
    return false;                                                      // 590
  };                                                                   //
                                                                       //
  logoutSandstorm = function () {                                      // 593
    Meteor.logout(function () {                                        // 594
      sessionStorage.removeItem("linkingIdentityLoginToken");          // 595
      Accounts._loginButtonsSession.closeDropdown();                   // 596
      globalTopbar.closePopup();                                       // 597
      var openGrains = globalGrains.get();                             // 598
      openGrains.forEach(function (grain) {                            // 599
        grain.destroy();                                               // 600
      });                                                              //
      globalGrains.set([]);                                            // 602
      Router.go("root");                                               // 603
    });                                                                //
  };                                                                   //
                                                                       //
  function makeAccountSettingsUi() {                                   // 607
    return new SandstormAccountSettingsUi(globalTopbar, globalDb, window.location.protocol + "//" + makeWildcardHost("static"));
  }                                                                    //
                                                                       //
  Template.layout.helpers({                                            // 612
    adminAlertIsTooLarge: function () {                                // 613
      Template.instance().resizeTracker.depend();                      // 614
      var setting = Settings.findOne({ _id: "adminAlert" });           // 615
      if (!setting || !setting.value) {                                // 616
        return false;                                                  // 617
      }                                                                //
      // TODO(someday): 10 and 850 are just magic values that estimate the size of the font and the
      // number of pixels everything else in the topbar respectively. This should really be
      // calculated based on actual sizes of things in the topbar.     //
      return window.innerWidth - setting.value.length * 10 < 850;      // 622
    },                                                                 //
    adminAlert: function () {                                          // 624
      var setting = Settings.findOne({ _id: "adminAlert" });           // 625
      if (!setting || !setting.value) {                                // 626
        return null;                                                   // 627
      }                                                                //
      var text = setting.value;                                        // 629
                                                                       //
      var alertTimeSetting = Settings.findOne({ _id: "adminAlertTime" });
      var alertTime = alertTimeSetting && alertTimeSetting.value;      // 632
                                                                       //
      var alertUrlSetting = Settings.findOne({ _id: "adminAlertUrl" });
      var alertUrl = alertUrlSetting ? alertUrlSetting.value.trim() : null;
      if (!alertUrl) alertUrl = null;                                  // 636
                                                                       //
      var template = Template.instance();                              // 638
      var param;                                                       // 639
      var className;                                                   // 640
      if (text.indexOf("$TIME") !== -1) {                              // 641
        if (!alertTime) return null;                                   // 642
        text = text.replace("$TIME", alertTime.toLocaleTimeString());  // 643
      }                                                                //
      if (text.indexOf("$DATE") !== -1) {                              // 645
        if (!alertTime) return null;                                   // 646
        text = text.replace("$DATE", alertTime.toLocaleDateString());  // 647
      }                                                                //
      if (text.indexOf("$IN_COUNTDOWN") !== -1) {                      // 649
        if (!alertTime) return null;                                   // 650
        param = formatInCountdown(template, alertTime);                // 651
        if (!param) return null;                                       // 652
        text = text.replace("$IN_COUNTDOWN", param.text);              // 653
        className = param.className;                                   // 654
      }                                                                //
      if (text.indexOf("$ACCOUNT_EXPIRES_IN") !== -1) {                // 656
        param = formatAccountExpiresIn(template);                      // 657
        if (!param) return null;                                       // 658
        text = text.replace("$ACCOUNT_EXPIRES_IN", param.text);        // 659
        className = param.className;                                   // 660
      }                                                                //
      if (text.indexOf("$ACCOUNT_EXPIRES") !== -1) {                   // 662
        param = formatAccountExpires();                                // 663
        if (!param) return null;                                       // 664
        text = text.replace("$ACCOUNT_EXPIRES", param);                // 665
      }                                                                //
      if (text.indexOf("$APPNAME") !== -1) {                           // 667
        text = text.replace("$APPNAME", determineAppName(this.grainId));
      }                                                                //
      if (alertUrl && alertUrl.indexOf("$APPNAME") !== -1) {           // 670
        alertUrl = alertUrl.replace("$APPNAME", determineAppName(this.grainId));
      }                                                                //
      return { text: text, className: className, alertUrl: alertUrl };
    },                                                                 //
    billingPromptState: function () {                                  // 675
      return billingPromptState.get();                                 // 676
    },                                                                 //
    demoExpired: isDemoExpired,                                        // 678
    canUpgradeDemo: function () {                                      // 679
      return Meteor.settings["public"].allowUninvited;                 // 680
    },                                                                 //
    globalAccountsUi: function () {                                    // 682
      return globalAccountsUi;                                         // 683
    },                                                                 //
    identityUser: function () {                                        // 685
      var user = Meteor.user();                                        // 686
      return user && user.profile;                                     // 687
    },                                                                 //
    showAccountButtons: function () {                                  // 689
      return Meteor.user() && !Meteor.loggingIn() && !isDemoUser();    // 690
    },                                                                 //
    accountButtonsData: function () {                                  // 692
      return { isAdmin: globalDb.isAdmin() };                          // 693
    },                                                                 //
    firstLogin: function () {                                          // 695
      return credentialsSubscription.ready() && !isDemoUser() && !Meteor.loggingIn() && Meteor.user() && !Meteor.user().hasCompletedSignup;
    },                                                                 //
    accountSettingsUi: function () {                                   // 699
      return makeAccountSettingsUi();                                  // 700
    },                                                                 //
    firstTimeBillingPromptState: function () {                         // 702
      // Should we show the first-time billing plan selector?          //
                                                                       //
      // Don't show if billing is not enabled.                         //
      if (!window.BlackrockPayments) return;                           // 706
                                                                       //
      var user = Meteor.user();                                        // 708
                                                                       //
      // Don't show if not logged in.                                  //
      if (!user) return;                                               // 711
                                                                       //
      // Don't show if not in the experiment.                          //
      if (!user.experiments || user.experiments.firstTimeBillingPrompt !== "test") return;
                                                                       //
      // Don't show if the user has selected a plan already.           //
      if (user.plan && !Session.get("firstTimeBillingPromptOpen")) return;
                                                                       //
      // Only show to account users (not identities).                  //
      if (!user.loginIdentities) return;                               // 720
                                                                       //
      // Don't show to demo users.                                     //
      if (user.expires) return;                                        // 723
                                                                       //
      // Don't show when viewing another user's grain. We don't want to scare people away from
      // logging in to collaborate.                                    //
      var route = Router.current().route.getName();                    // 727
      if (route === "shared") return;                                  // 728
      if (route === "grain") {                                         // 729
        if (_.some(globalGrains.get(), function (grain) {              // 730
          return grain.isActive() && !grain.isOwner();                 // 731
        })) {                                                          //
          return;                                                      // 733
        }                                                              //
      }                                                                //
                                                                       //
      // Don't let the plan chooser disappear instantly once user.plan is set.
      Session.set("firstTimeBillingPromptOpen", true);                 // 738
                                                                       //
      // OK, show it.                                                  //
      return {                                                         // 741
        db: globalDb,                                                  // 742
        topbar: globalTopbar,                                          // 743
        accountsUi: globalAccountsUi,                                  // 744
        onComplete: function () {                                      // 745
          Session.set("firstTimeBillingPromptOpen", false);            // 746
        }                                                              //
      };                                                               //
    }                                                                  //
  });                                                                  //
                                                                       //
  Template.layout.events({                                             // 752
    "click #demo-expired .logout": function (event) {                  // 753
      logoutSandstorm();                                               // 754
    }                                                                  //
  });                                                                  //
                                                                       //
  Template.root.events({                                               // 758
    "click #logo": function (event) {                                  // 759
      doLogoAnimation(event.shiftKey, 0);                              // 760
    }                                                                  //
  });                                                                  //
                                                                       //
  credentialsSubscription = Meteor.subscribe("credentials");           // 764
                                                                       //
  makeDateString = function (date) {                                   // 766
    // Note: this is also used by grain.js.                            //
                                                                       //
    if (!date) {                                                       // 769
      return "";                                                       // 770
    }                                                                  //
                                                                       //
    var result;                                                        // 773
                                                                       //
    var now = new Date();                                              // 775
    var diff = now.valueOf() - date.valueOf();                         // 776
                                                                       //
    if (diff < 86400000 && now.getDate() === date.getDate()) {         // 778
      result = date.toLocaleTimeString();                              // 779
    } else {                                                           //
      result = MONTHS[date.getMonth()] + " " + date.getDate() + " ";   // 781
                                                                       //
      if (now.getFullYear() !== date.getFullYear()) {                  // 783
        result = date.getFullYear() + " " + result;                    // 784
      }                                                                //
    }                                                                  //
                                                                       //
    return result;                                                     // 788
  };                                                                   //
  Template.registerHelper("dateString", makeDateString);               // 790
  Template.registerHelper("hideNavbar", function () {                  // 791
    // Hide navbar if user is not logged in, since they can't go anywhere with it.
    return !Meteor.userId() || isDemoExpired();                        // 793
  });                                                                  //
  Template.registerHelper("shrinkNavbar", function () {                // 795
    // Shrink the navbar if the user clicked the button to do so.      //
    return Session.get("shrink-navbar");                               // 797
  });                                                                  //
                                                                       //
  Template.registerHelper("quotaEnabled", function () {                // 800
    return Meteor.settings["public"].quotaEnabled;                     // 801
  });                                                                  //
                                                                       //
  prettySize = function (size) {                                       // 804
    var suffix = "B";                                                  // 805
    if (size >= 1000000000) {                                          // 806
      size = size / 1000000000;                                        // 807
      suffix = "GB";                                                   // 808
    } else if (size >= 1000000) {                                      //
      size = size / 1000000;                                           // 810
      suffix = "MB";                                                   // 811
    } else if (size >= 1000) {                                         //
      size = size / 1000;                                              // 813
      suffix = "kB";                                                   // 814
    }                                                                  //
    return size.toPrecision(3) + suffix;                               // 816
  };                                                                   //
                                                                       //
  launchAndEnterGrainByPackageId = function (packageId) {              // 819
    var action = UserActions.findOne({ packageId: packageId });        // 820
    if (!action) {                                                     // 821
      alert("Somehow, you seem to have attempted to launch a package you have not installed.");
      return;                                                          // 823
    } else {                                                           //
      launchAndEnterGrainByActionId(action._id, null, null);           // 825
    }                                                                  //
  };                                                                   //
                                                                       //
  launchAndEnterGrainByActionId = function (actionId, devPackageId, devIndex) {
    // Note that this takes a devPackageId and a devIndex as well. If provided,
    // they override the actionId.                                     //
    var packageId;                                                     // 832
    var command;                                                       // 833
    var appTitle;                                                      // 834
    var nounPhrase;                                                    // 835
    if (devPackageId) {                                                // 836
      var devPackage = DevPackages.findOne(devPackageId);              // 837
      if (!devPackage) {                                               // 838
        console.error("no such dev package: ", devPackageId);          // 839
        return;                                                        // 840
      }                                                                //
      var devAction = devPackage.manifest.actions[devIndex];           // 842
      packageId = devPackageId;                                        // 843
      command = devAction.command;                                     // 844
      appTitle = SandstormDb.appNameFromPackage(devPackage);           // 845
      nounPhrase = SandstormDb.nounPhraseForActionAndAppTitle(devAction, appTitle);
    } else {                                                           //
      var action = UserActions.findOne(actionId);                      // 848
      if (!action) {                                                   // 849
        console.error("no such action:", actionId);                    // 850
        return;                                                        // 851
      }                                                                //
                                                                       //
      packageId = action.packageId;                                    // 854
      var pkg = Packages.findOne(packageId);                           // 855
      command = action.command;                                        // 856
      appTitle = SandstormDb.appNameFromPackage(pkg);                  // 857
      nounPhrase = SandstormDb.nounPhraseForActionAndAppTitle(action, appTitle);
    }                                                                  //
                                                                       //
    var title = "Untitled " + appTitle + " " + nounPhrase;             // 861
                                                                       //
    var identityId = Accounts.getCurrentIdentityId();                  // 863
                                                                       //
    // We need to ask the server to start a new grain, then browse to it.
    Meteor.call("newGrain", packageId, command, title, identityId, function (error, grainId) {
      if (error) {                                                     // 867
        console.error(error);                                          // 868
        alert(error.message);                                          // 869
      } else {                                                         //
        Router.go("grain", { grainId: grainId });                      // 871
      }                                                                //
    });                                                                //
  };                                                                   //
                                                                       //
  Template.root.helpers({                                              // 876
    storageUsage: function () {                                        // 877
      return Meteor.userId() ? prettySize(Meteor.user().storageUsage || 0) : undefined;
    },                                                                 //
                                                                       //
    storageQuota: function () {                                        // 881
      var plan = globalDb.getMyPlan();                                 // 882
      return plan ? prettySize(plan.storage) : undefined;              // 883
    },                                                                 //
                                                                       //
    overQuota: function () {                                           // 886
      return !window.BlackrockPayments && isUserOverQuota(Meteor.user());
    }                                                                  //
  });                                                                  //
                                                                       //
  Template.root.events({                                               // 891
    "click .uninstall-app-button": function (event) {                  // 892
      // TODO(cleanup): This event handler is no longer used, but the new UI does not yet implement
      //   uninstall. Leave this code here for reference until it does.
      var appId = event.currentTarget.getAttribute("data-appid");      // 895
      if (window.confirm("Really uninstall this app?")) {              // 896
        UserActions.find({ appId: appId, userId: Meteor.userId() }).forEach(function (action) {
          UserActions.remove(action._id);                              // 898
        });                                                            //
        Meteor.call("deleteUnusedPackages", appId);                    // 900
      }                                                                //
    }                                                                  //
  });                                                                  //
                                                                       //
  Template.notificationsPopup.helpers({                                // 905
    notifications: function () {                                       // 906
      Meteor.call("readAllNotifications");                             // 907
      return Notifications.find({ userId: Meteor.userId() }, { sort: { timestamp: -1 } }).map(function (row) {
        var grain = Grains.findOne({ _id: row.grainId });              // 909
        if (grain) {                                                   // 910
          row.grainTitle = grain.title;                                // 911
        }                                                              //
        return row;                                                    // 913
      });                                                              //
    }                                                                  //
  });                                                                  //
                                                                       //
  Template.notifications.helpers({                                     // 918
    notificationCount: function () {                                   // 919
      return Notifications.find({ userId: Meteor.userId(), isUnread: true }).count();
    }                                                                  //
  });                                                                  //
                                                                       //
  Template.notificationsPopup.events({                                 // 924
    "click #notification-dropdown": function (event) {                 // 925
      return false;                                                    // 926
    }                                                                  //
  });                                                                  //
                                                                       //
  Template.notificationItem.helpers({                                  // 930
    isAppUpdates: function () {                                        // 931
      return !!this.appUpdates;                                        // 932
    },                                                                 //
    notificationTitle: function () {                                   // 934
      if (this.admin) {                                                // 935
        return "Notification from System";                             // 936
      } else if (this.appUpdates) {                                    //
        return "App updates are available";                            // 938
      } else if (this.referral) {                                      //
        return false;                                                  // 940
      }                                                                //
                                                                       //
      return this.grainTitle + " is backgrounded";                     // 943
    },                                                                 //
    titleHelperText: function () {                                     // 945
      if (this.admin) {                                                // 946
        return "Dismiss this system notification";                     // 947
      }                                                                //
                                                                       //
      return "Stops the background app";                               // 950
    },                                                                 //
    dismissText: function () {                                         // 952
      if (this.admin && this.admin.type === "reportStats") {           // 953
        return false;                                                  // 954
      } else if (this.referral) {                                      //
        return "Dismiss";                                              // 956
      }                                                                //
      return "Cancel";                                                 // 958
    },                                                                 //
    adminLink: function () {                                           // 960
      return this.admin && this.admin.action;                          // 961
    },                                                                 //
    appUpdatesList: function () {                                      // 963
      return _.values(this.appUpdates);                                // 964
    },                                                                 //
    paidUser: function () {                                            // 966
      var plan = Meteor.user().plan;                                   // 967
      return plan && plan !== "free";                                  // 968
    }                                                                  //
  });                                                                  //
                                                                       //
  Template.notificationItem.events({                                   // 972
    "click .cancel-notification": function (event) {                   // 973
      Meteor.call("dismissNotification", this._id);                    // 974
      return false;                                                    // 975
    },                                                                 //
    "click .accept-notification": function (event) {                   // 977
      if (this.appUpdates) {                                           // 978
        var self = this;                                               // 979
        Meteor.call("updateApps", this.appUpdates, function (err) {    // 980
          // TODO(someday): if (err)                                   //
          Meteor.call("dismissNotification", self._id);                // 982
        });                                                            //
      }                                                                //
      return false;                                                    // 985
    },                                                                 //
    "click .dismiss-notification": function (event) {                  // 987
      Meteor.call("dismissNotification", this._id);                    // 988
    }                                                                  //
  });                                                                  //
                                                                       //
  Meteor.startup(function () {                                         // 992
    // Tell app authors how to run JS in the context of the grain-frame.
    console.log("%cApp authors: To understand the grain-frame in Sandstorm and how to find " + "logs and perform troubleshooting, see: " + "\n- https://docs.sandstorm.io/en/latest/developing/path/ " + "\n- https://docs.sandstorm.io/en/latest/using/top-bar/ " + "\n- https://docs.sandstorm.io/en/latest/developing/troubleshooting/ " + "\n" + "\nWhen debugging, make sure you execute Javascript " + "in the context of the 'grain-frame' IFRAME. References: " + "\n- https://stackoverflow.com/questions/3275816/debugging-iframes-with-chrome-developer-tools " + "\n- https://developer.mozilla.org/en-US/docs/Tools/Working_with_iframes " + "\n" + "\nWe can also provide personal assistance! Get in touch: https://sandstorm.io/community", "font-size: large; background-color: yellow;");
                                                                       //
    var self = this;                                                   // 1009
    Meteor.subscribe("notifications");                                 // 1010
                                                                       //
    Meteor.autorun(function () {                                       // 1012
      Meteor.subscribe("notificationGrains", Notifications.find().map(function (row) {
        return row._id;                                                // 1015
      }));                                                             //
    });                                                                //
  });                                                                  //
}                                                                      //
Router.configure({                                                     // 1021
  layoutTemplate: 'layout',                                            // 1022
  notFoundTemplate: "notFound",                                        // 1023
  loadingTemplate: "loading"                                           // 1024
});                                                                    //
                                                                       //
if (Meteor.isClient) {                                                 // 1027
  Router.onBeforeAction("loading");                                    // 1028
}                                                                      //
                                                                       //
function getBuildInfo() {                                              // 1031
  var build = Meteor.settings && Meteor.settings["public"] && Meteor.settings["public"].build;
  var isNumber = typeof build === "number";                            // 1033
  if (!build) {                                                        // 1034
    build = "(unknown)";                                               // 1035
  } else if (isNumber) {                                               //
    build = String(Math.floor(build / 1000)) + "." + String(build % 1000);
  }                                                                    //
  return {                                                             // 1039
    build: build,                                                      // 1040
    isUnofficial: !isNumber                                            // 1041
  };                                                                   //
}                                                                      //
                                                                       //
var promptForFile = function (input, callback) {                       // 1045
  // TODO(cleanup): Share code with "upload picture" and other upload buttons.
  function listener(e) {                                               // 1047
    input.removeEventListener("change", listener);                     // 1048
    callback(e.currentTarget.files[0]);                                // 1049
  }                                                                    //
  input.addEventListener("change", listener);                          // 1051
  input.click();                                                       // 1052
};                                                                     //
                                                                       //
var startUpload = function (file, endpoint, onComplete) {              // 1055
  // TODO(cleanup): Use Meteor's HTTP, although this may require sending them a PR to support
  //   progress callbacks (and officially document that binary input is accepted).
                                                                       //
  Session.set("uploadStatus", "Uploading");                            // 1059
                                                                       //
  var xhr = new XMLHttpRequest();                                      // 1061
                                                                       //
  xhr.onreadystatechange = function () {                               // 1063
    if (xhr.readyState == 4) {                                         // 1064
      console.log(xhr.status);                                         // 1065
      if (xhr.status == 200) {                                         // 1066
        Session.set("uploadProgress", 0);                              // 1067
        onComplete(xhr.responseText);                                  // 1068
      } else {                                                         //
        Session.set("uploadError", {                                   // 1070
          status: xhr.status,                                          // 1071
          statusText: xhr.statusText,                                  // 1072
          response: xhr.responseText                                   // 1073
        });                                                            //
      }                                                                //
    }                                                                  //
  };                                                                   //
                                                                       //
  if (xhr.upload) {                                                    // 1079
    xhr.upload.addEventListener("progress", function (progressEvent) {
      Session.set("uploadProgress", Math.floor(progressEvent.loaded / progressEvent.total * 100));
    });                                                                //
  }                                                                    //
                                                                       //
  xhr.open("POST", endpoint, true);                                    // 1086
  xhr.send(file);                                                      // 1087
                                                                       //
  Router.go("uploadStatus");                                           // 1089
};                                                                     //
                                                                       //
restoreBackup = function (file) {                                      // 1092
  // This function is global so tests can call it                      //
  startUpload(file, "/uploadBackup", function (response) {             // 1094
    Session.set("uploadStatus", "Unpacking");                          // 1095
    var identityId = Accounts.getCurrentIdentityId();                  // 1096
    Meteor.call("restoreGrain", response, identityId, function (err, grainId) {
      if (err) {                                                       // 1098
        console.log(err);                                              // 1099
        Session.set("uploadStatus", undefined);                        // 1100
        Session.set("uploadError", {                                   // 1101
          status: "",                                                  // 1102
          statusText: err.message                                      // 1103
        });                                                            //
      } else {                                                         //
        Router.go("grain", { grainId: grainId });                      // 1106
      }                                                                //
    });                                                                //
  });                                                                  //
};                                                                     //
                                                                       //
promptRestoreBackup = function (input) {                               // 1112
  promptForFile(input, restoreBackup);                                 // 1113
};                                                                     //
                                                                       //
uploadApp = function (file) {                                          // 1116
  // This function is global so tests can call it                      //
  Meteor.call("newUploadToken", function (err, token) {                // 1118
    if (err) {                                                         // 1119
      console.error(err);                                              // 1120
      alert(err.message);                                              // 1121
    } else {                                                           //
      startUpload(file, "/upload/" + token, function (response) {      // 1123
        Session.set("uploadStatus", undefined);                        // 1124
        Router.go("install", { packageId: response });                 // 1125
      });                                                              //
    }                                                                  //
  });                                                                  //
};                                                                     //
                                                                       //
promptUploadApp = function (input) {                                   // 1131
  promptForFile(input, uploadApp);                                     // 1132
};                                                                     //
                                                                       //
Router.map(function () {                                               // 1135
  this.route("root", {                                                 // 1136
    path: "/",                                                         // 1137
    waitOn: function () {                                              // 1138
      return [Meteor.subscribe("hasUsers"), Meteor.subscribe("grainsMenu")];
    },                                                                 //
    data: function () {                                                // 1144
      // If the user is logged-in, and can create new grains, and      //
      // has no grains yet, then send them to "new".                   //
      if (this.ready() && Meteor.userId() && !Meteor.loggingIn()) {    // 1147
        if (globalDb.currentUserGrains({}, {}).count() === 0 && globalDb.currentUserApiTokens().count() === 0) {
          Router.go("apps", {}, { replaceState: true });               // 1150
        } else {                                                       //
          Router.go("grains", {}, { replaceState: true });             // 1152
        }                                                              //
      }                                                                //
                                                                       //
      return {                                                         // 1156
        needsAdminTokenLogin: this.ready() && !HasUsers.findOne("hasUsers") && !globalDb.allowDevAccounts(),
        build: getBuildInfo().build,                                   // 1158
        splashUrl: (Settings.findOne("splashUrl") || {}).value         // 1159
      };                                                               //
    }                                                                  //
  });                                                                  //
                                                                       //
  this.route("linkHandler", {                                          // 1164
    path: "/link-handler/:url",                                        // 1165
                                                                       //
    data: function () {                                                // 1167
      var url = this.params.url;                                       // 1168
      if (url.lastIndexOf("web+sandstorm:", 0) === 0) {                // 1169
        url = url.slice("web+sandstorm:".length);                      // 1170
      }                                                                //
      // TODO(cleanup):  Didn't use Router.go() because the url may contain a query term.
      document.location = "/install/" + url;                           // 1173
      return {};                                                       // 1174
    }                                                                  //
  });                                                                  //
                                                                       //
  this.route("about", {                                                // 1178
    path: "/about",                                                    // 1179
    data: function () {                                                // 1180
      var result = getBuildInfo();                                     // 1181
                                                                       //
      var backers = Session.get("backers");                            // 1183
      if (backers) {                                                   // 1184
        result.backers = backers.names;                                // 1185
        result.anonCount = backers.anonCount;                          // 1186
      } else {                                                         //
        HTTP.get("/sandstorm-backers.txt", function (err, response) {  // 1188
          var names = response.content.split("\n").sort(function (a, b) {
            return a.toLowerCase().localeCompare(b.toLowerCase());     // 1190
          });                                                          //
                                                                       //
          var anonCount = 0;                                           // 1193
          while (anonCount < names.length && names[anonCount] === "") {
            ++anonCount;                                               // 1195
          }                                                            //
          names = names.slice(anonCount);                              // 1197
                                                                       //
          // File ends in trailing newline, but that last blank line does not represent an
          // anonymous contributor.                                    //
          --anonCount;                                                 // 1201
                                                                       //
          Session.set("backers", { names: names, anonCount: anonCount });
        });                                                            //
      }                                                                //
                                                                       //
      result.termsUrl = globalDb.getSetting("termsUrl");               // 1207
      result.privacyUrl = globalDb.getSetting("privacyUrl");           // 1208
                                                                       //
      return result;                                                   // 1210
    }                                                                  //
  });                                                                  //
                                                                       //
  this.route("uploadStatus", {                                         // 1214
    path: "/upload",                                                   // 1215
                                                                       //
    waitOn: function () {                                              // 1217
      return Meteor.subscribe("credentials");                          // 1218
    },                                                                 //
                                                                       //
    data: function () {                                                // 1221
      return {                                                         // 1222
        progress: Session.get("uploadProgress"),                       // 1223
        status: Session.get("uploadStatus"),                           // 1224
        error: Session.get("uploadError")                              // 1225
      };                                                               //
    }                                                                  //
  });                                                                  //
                                                                       //
  this.route("referrals", {                                            // 1230
    path: "/referrals",                                                // 1231
                                                                       //
    waitOn: function () {                                              // 1233
      return Meteor.subscribe("referralInfoPseudo");                   // 1234
    }                                                                  //
  });                                                                  //
                                                                       //
  this.route("account", {                                              // 1238
    path: "/account",                                                  // 1239
                                                                       //
    data: function () {                                                // 1241
      // Don't allow logged-out or demo users to visit the accounts page. There should be no way
      // for them to get there except for typing the URL manually. In theory showing the accounts
      // page to a demo user could make some sense for editing their profile, but we really do not
      // want them signing up for subscription plans!                  //
      if (!Meteor.user() && !Meteor.loggingIn() || globalDb.isDemoUser()) {
        Router.go("root", {}, { replaceState: true });                 // 1247
      } else {                                                         //
        return makeAccountSettingsUi();                                // 1249
      }                                                                //
    }                                                                  //
  });                                                                  //
                                                                       //
  this.route("accountUsage", {                                         // 1254
    path: "/account/usage"                                             // 1255
  });                                                                  //
});                                                                    //
/////////////////////////////////////////////////////////////////////////

}).call(this);

//# sourceMappingURL=shell.js.map
